@@ -3,8 +3,6 @@
 
 /* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2009      */
 
-// $Id: bldcfg.c 5562 2009-12-24 14:29:38Z rbowler $
-
 /*-------------------------------------------------------------------*/
 /* This module builds the configuration tables for the Hercules      */
 /* ESA/390 emulator.  It reads information about the processors      */
@@ -824,6 +822,11 @@ char    pathname[MAX_PATH];             /* file path in host format  */
     ptt_trace_init (0, 1);
 #endif
 
+#if defined(_FEATURE_MESSAGE_SECURITY_ASSIST)
+    /* Initialize the wrapping key registers lock */
+    initialize_lock(&sysblk.wklock);
+#endif /*defined(_FEATURE_MESSAGE_SECURITY_ASSIST)*/
+
 #if defined(OPTION_FISHIO)
     InitIOScheduler                     // initialize i/o scheduler...
     (
