@@ -1,54 +1,18 @@
-/* FEATCHK.H    (c) Copyright Jan Jaeger, 2000-2007                  */
+/* FEATCHK.H    (c) Copyright Jan Jaeger, 2000-2009                  */
+/*              Feature definition consistency checks                */
+
 /*-------------------------------------------------------------------*/
-/*                                                                   */
-/*           Feature definition consistency checks                   */
-/*                                                                   */
 /*  Perform various checks on feature combinations, and set          */
 /*  additional flags to percolate certain features such as           */
 /*  SIE down to lower architecture levels such that these            */
 /*  can include emulation support                                    */
 /*                                                                   */
+/*  FEATURE_XXXX is defined per architecture mode, and               */
+/*  _FEATURE_XXXX is defined across all architectures                */
+/*  if FEATURE_ XXXX is defined for any architecture mode.           */
+/*                                                                   */
 /*-------------------------------------------------------------------*/
 
-// $Id: featchk.h,v 1.51 2008/12/27 23:34:37 rbowler Exp $
-//
-// $Log: featchk.h,v $
-// Revision 1.51  2008/12/27 23:34:37  rbowler
-// Integrated 3270 (SYSG) console send command
-//
-// Revision 1.50  2008/12/24 22:35:53  rbowler
-// Framework for integrated 3270 and ASCII console features
-//
-// Revision 1.49  2008/12/22 00:29:10  ivan
-// Implement February 2008 z/Arch Compare And Swap And Store Facility 2
-// Update FAQ to reflect change
-// update FAQ to also indicate z/Arch DAT Enhancement is implemented
-//
-// Revision 1.48  2007/12/23 00:29:20  rbowler
-// reset_channel_path, cancel_subchannel unresolved if no channel subsystem feature
-//
-// Revision 1.47  2007/06/23 00:04:09  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.46  2007/03/08 01:27:02  gsmith
-// Remove inline attr from vfetchx/vstorex _full functions
-//
-// Revision 1.45  2007/01/14 23:31:46  gsmith
-// nerak's patch, one more time
-//
-// Revision 1.44  2007/01/14 19:49:13  gsmith
-// Fix typos
-//
-// Revision 1.43  2007/01/14 19:42:38  gsmith
-// Fix S370 only build - nerak60510
-//
-// Revision 1.42  2007/01/04 14:21:27  rbowler
-// Decimal floating point does not require binary floating point
-//
-// Revision 1.41  2006/12/08 09:43:21  jj
-// Add CVS message log
-//
-
 #if defined(FEATCHK_CHECK_ALL)
 
 /* FEATURE_INTERPRETIVE_EXECUTION is related to host related issues
@@ -158,6 +122,174 @@
  #define _FEATURE_INTEGRATED_ASCII_CONSOLE
 #endif
 
+#if defined(FEATURE_ESAME)
+ #define _FEATURE_ESAME
+#endif
+
+#if defined(FEATURE_ESAME_N3_ESA390)
+ #define _FEATURE_ESAME_N3_ESA390
+#endif
+
+#if defined(FEATURE_DAT_ENHANCEMENT)
+ #define _FEATURE_DAT_ENHANCEMENT
+#endif
+
+#if defined(FEATURE_STORE_FACILITY_LIST_EXTENDED)
+ #define _FEATURE_STORE_FACILITY_LIST_EXTENDED
+#endif
+
+#if defined(FEATURE_ENHANCED_DAT_FACILITY)
+ #define _FEATURE_ENHANCED_DAT_FACILITY
+#endif
+
+#if defined(FEATURE_SENSE_RUNNING_STATUS)
+ #define _FEATURE_SENSE_RUNNING_STATUS
+#endif
+
+#if defined(FEATURE_CONDITIONAL_SSKE)
+ #define _FEATURE_CONDITIONAL_SSKE
+#endif
+
+#if defined(FEATURE_CONFIGURATION_TOPOLOGY_FACILITY)
+ #define _FEATURE_CONFIGURATION_TOPOLOGY_FACILITY
+#endif
+
+#if defined(FEATURE_IPTE_RANGE_FACILITY)
+ #define _FEATURE_IPTE_RANGE_FACILITY
+#endif
+
+#if defined(FEATURE_NONQUIESCING_KEY_SETTING_FACILITY)
+ #define _FEATURE_NONQUIESCING_KEY_SETTING_FACILITY
+#endif
+
+#if defined(FEATURE_EXTENDED_TRANSLATION_FACILITY_2)
+ #define _FEATURE_EXTENDED_TRANSLATION_FACILITY_2
+#endif
+
+#if defined(FEATURE_MESSAGE_SECURITY_ASSIST)
+ #define _FEATURE_MESSAGE_SECURITY_ASSIST
+#endif
+
+#if defined(FEATURE_LONG_DISPLACEMENT)
+ #define _FEATURE_LONG_DISPLACEMENT
+#endif
+
+#if defined(FEATURE_HFP_MULTIPLY_ADD_SUBTRACT)
+ #define _FEATURE_HFP_MULTIPLY_ADD_SUBTRACT
+#endif
+
+#if defined(FEATURE_EXTENDED_IMMEDIATE)
+ #define _FEATURE_EXTENDED_IMMEDIATE
+#endif
+
+#if defined(FEATURE_EXTENDED_TRANSLATION_FACILITY_3)
+ #define _FEATURE_EXTENDED_TRANSLATION_FACILITY_3
+#endif
+
+#if defined(FEATURE_HFP_UNNORMALIZED_EXTENSION)
+ #define _FEATURE_HFP_UNNORMALIZED_EXTENSION
+#endif
+
+#if defined(FEATURE_ETF2_ENHANCEMENT)
+ #define _FEATURE_ETF2_ENHANCEMENT
+#endif
+
+#if defined(FEATURE_STORE_CLOCK_FAST)
+ #define _FEATURE_STORE_CLOCK_FAST
+#endif
+
+#if defined(FEATURE_MOVE_WITH_OPTIONAL_SPECIFICATIONS)
+ #define _FEATURE_MOVE_WITH_OPTIONAL_SPECIFICATIONS
+#endif
+
+#if defined(FEATURE_TOD_CLOCK_STEERING)
+ #define _FEATURE_TOD_CLOCK_STEERING
+#endif
+
+#if defined(FEATURE_ETF3_ENHANCEMENT)
+ #define _FEATURE_ETF3_ENHANCEMENT
+#endif
+
+#if defined(FEATURE_EXTRACT_CPU_TIME)
+ #define _FEATURE_EXTRACT_CPU_TIME
+#endif
+
+#if defined(FEATURE_COMPARE_AND_SWAP_AND_STORE)
+ #define _FEATURE_COMPARE_AND_SWAP_AND_STORE
+#endif
+
+#if defined(FEATURE_COMPARE_AND_SWAP_AND_STORE_FACILITY_2)
+ #define _FEATURE_COMPARE_AND_SWAP_AND_STORE_FACILITY_2
+#endif
+
+#if defined(FEATURE_GENERAL_INSTRUCTIONS_EXTENSION_FACILITY)
+ #define _FEATURE_GENERAL_INSTRUCTIONS_EXTENSION_FACILITY
+#endif
+
+#if defined(FEATURE_EXECUTE_EXTENSIONS_FACILITY)
+ #define _FEATURE_EXECUTE_EXTENSIONS_FACILITY
+#endif
+
+#if defined(FEATURE_ENHANCED_MONITOR_FACILITY)
+ #define _FEATURE_ENHANCED_MONITOR_FACILITY
+#endif
+
+#if defined(FEATURE_LOAD_PROGRAM_PARAMETER_FACILITY)
+ #define _FEATURE_LOAD_PROGRAM_PARAMETER_FACILITY
+#endif
+
+#if defined(FEATURE_FPS_ENHANCEMENT)
+ #define _FEATURE_FPS_ENHANCEMENT
+#endif
+
+#if defined(FEATURE_DECIMAL_FLOATING_POINT)
+ #define _FEATURE_DECIMAL_FLOATING_POINT
+#endif
+
+#if defined(FEATURE_PFPO)
+ #define _FEATURE_PFPO
+#endif
+
+#if defined(FEATURE_FAST_BCR_SERIALIZATION_FACILITY)
+ #define _FEATURE_FAST_BCR_SERIALIZATION_FACILITY
+#endif
+
+#if defined(FEATURE_RESET_REFERENCE_BITS_MULTIPLE_FACILITY)
+ #define _FEATURE_RESET_REFERENCE_BITS_MULTIPLE_FACILITY
+#endif
+
+#if defined(FEATURE_CPU_MEASUREMENT_COUNTER_FACILITY)
+ #define _FEATURE_CPU_MEASUREMENT_COUNTER_FACILITY
+#endif
+
+#if defined(FEATURE_CPU_MEASUREMENT_SAMPLING_FACILITY)
+ #define _FEATURE_CPU_MEASUREMENT_SAMPLING_FACILITY
+#endif
+
+#if defined(FEATURE_ACCESS_EXCEPTION_FETCH_STORE_INDICATION)
+ #define _FEATURE_ACCESS_EXCEPTION_FETCH_STORE_INDICATION
+#endif
+
+#if defined(FEATURE_MESSAGE_SECURITY_ASSIST_EXTENSION_3)
+ #define _FEATURE_MESSAGE_SECURITY_ASSIST_EXTENSION_3
+#endif
+
+#if defined(FEATURE_MESSAGE_SECURITY_ASSIST_EXTENSION_4)
+ #define _FEATURE_MESSAGE_SECURITY_ASSIST_EXTENSION_4
+#endif
+
+#if defined(FEATURE_MESSAGE_SECURITY_ASSIST_EXTENSION_1)
+ #define _FEATURE_MESSAGE_SECURITY_ASSIST_EXTENSION_1
+#endif
+
+#if defined(FEATURE_MESSAGE_SECURITY_ASSIST_EXTENSION_2)
+ #define _FEATURE_MESSAGE_SECURITY_ASSIST_EXTENSION_2
+#endif
+
+#if defined(FEATURE_HERCULES_DIAGCALLS)
+ #define _FEATURE_HERCULES_DIAGCALLS
+#endif
+
 #undef _VSTORE_C_STATIC
 #if !defined(OPTION_NO_INLINE_VSTORE)
  #define _VSTORE_C_STATIC static inline
@@ -291,6 +423,11 @@
  #error Move page facility cannot be defined with 2K storage keys
 #endif
 
+#if defined(FEATURE_FAST_SYNC_DATA_MOVER) \
+ && !defined(FEATURE_MOVE_PAGE_FACILITY_2)
+ #error Fast sync data mover facility requires Move page facility
+#endif
+
 #if defined(FEATURE_ESAME) \
   && defined(FEATURE_INTERPRETIVE_EXECUTION) \
   && !defined(_FEATURE_SIE)
@@ -404,6 +541,11 @@
  #error HFP unnormalized extension requires hexadecimal floating point
 #endif
 
+#if defined(FEATURE_FLOATING_POINT_EXTENSION_FACILITY) \
+ && !defined(FEATURE_BINARY_FLOATING_POINT)
+ #error Floating point extension facility requires binary floating point
+#endif
+
 #if defined(FEATURE_PER2) && !defined(FEATURE_PER)
  #error FEATURE_PER must be defined when using FEATURE_PER2
 #endif
@@ -424,6 +566,9 @@
  #error Integrated ASCII console requires FEATURE_SYSTEM_CONSOLE
 #endif
 
+#if defined(FEATURE_VM_BLOCKIO) && !defined(FEATURE_EMULATE_VM)
+ #error VM Standard Block I/O DIAGNOSE 0x250 requires FEATURE_EMULATE_VM
+#endif
 
 #if defined(FEATURE_MESSAGE_SECURITY_ASSIST)
  #if defined(_370)
@@ -437,6 +582,30 @@
  #endif
 #endif /*defined(FEATURE_MESSAGE_SECURITY_ASSIST)*/
 
+#if defined(FEATURE_STORE_FACILITY_LIST_EXTENDED)
+  #if !defined(FEATURE_STORE_FACILITY_LIST)
+    #error STFLE requires STFL (FEATURE_STORE_FACILITY_LIST_EXTENDED requires FEATURE_STORE_FACILITY_LIST)
+  #endif
+#endif
+
+#if defined(FEATURE_ACCESS_EXCEPTION_FETCH_STORE_INDICATION)         /*810*/
+  #if !defined(FEATURE_ENHANCED_SUPPRESSION_ON_PROTECTION)
+    #error Access-Exception Fetch/Store Indication facility requires Enhanced Suppression on Protection
+  #endif
+#endif
+
+#if defined(FEATURE_ENHANCED_SUPPRESSION_ON_PROTECTION)              /*208*/
+  #if !defined(FEATURE_SUPPRESSION_ON_PROTECTION)
+    #error Enhanced Suppression on Protection facility requires Suppression on Protection
+  #endif
+#endif
+
+#if defined(FEATURE_CPU_MEASUREMENT_COUNTER_FACILITY)\
+    || defined(FEATURE_CPU_MEASUREMENT_SAMPLING_FACILITY)
+  #if !defined(FEATURE_LOAD_PROGRAM_PARAMETER_FACILITY)
+    #error CPU Measurement facilities requires Load Program Parameter facility 
+  #endif
+#endif
 
 #endif /*!defined(FEATALL_CHECKALL)*/
 
